// workspace.c : Main source file for the parser implementation
//
#ifdef _WIN32
#include "stdafx.h"
#endif
#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include <BipCpi.h>
#include <BipCni.h>
#include "ParserUtils.h"
#include "NodeUtils.h"
#ifdef _WIN32
#include "NT_msg/workspace_msg.h"
#else
#include "AIX_msg/MQSIV2_WKSPACE_msg.h"
#endif



CciChar* ASCIIToUnicode(const char* aStr);

#ifdef _WIN32
void __declspec(dllexport) *bipGetParserFactory();

static HMODULE g_module;
BOOL APIENTRY DllMain( HANDLE hModule, 
                       DWORD  ul_reason_for_call, 
                       LPVOID lpReserved
					 )
{
	g_module = hModule;
    return TRUE;
}
#endif

///////////////////////////////////////////////////////////
// ErrorToString
char *ErrorToString(int iRc)
{
	static char	message[256];
	switch(iRc)
	{
	case CCI_PARSER_NAME_TOO_LONG:
		return("Parse name too long");
	default:
		sprintf(message, "Error: Unknown code: %d", iRc);
		return(message);
	}
}


///////////////////////////////////////////////////////////
// CreateContext
void *CreateContext(CciParser* parser)
{
	mqsixLog(">> CreateContext\n");
	mqsixLog("<< CreateContext\n");
	return(NULL);
}

///////////////////////////////////////////////////////////
// Delete Context
void DeleteContext(CciParser*  parser, CciContext* context)
{
	mqsixLog(">> Entered DeleteContext\n");
	if (context != NULL)
	{
		free(context);
	}
	mqsixLog("<< Entered DeleteContext\n");
}

///////////////////////////////////////////////////////////
// ParseBuffer
int ParseBuffer(CciParser	*parser,
			    CciContext	*context,
			    int         iOffset)
{
	int		iRc;
	CciChar	*pMessageSource;
	pMessageSource = ASCIIToUnicode("MQSIV2_WKSPACE");
	cciThrowException(
		&iRc,						// Return code
		CCI_PARSER_EXCEPTION,		// Type of exception
		__FILE__,					// File name
		__LINE__,					// Exception source line number
		"ParseBuffer",				// Function that threw exception
		pMessageSource,				// Message source
		MSG_INVALID_PARSER_USAGE,	// Message number
		"Attempt to use WKSPACE to parse incoming data",
		(char *)0					// Parameters
		);
	free(pMessageSource);
	return(0);
}

///////////////////////////////////////////////////////////
// ParseFirstChild
void ParseFirstChild(CciParser	*parser,
					 CciContext	*context,
					 CciElement	*currentElement)
{
	int		iRc;
	CciElement	*rootElement;
	mqsixLog(">> ParseFirstChild() element=%p\n", currentElement);
	rootElement = cpiRootElement(&iRc, parser);
	cpiSetElementCompleteNext(&iRc, rootElement, 1);
	cpiSetElementCompletePrevious(&iRc, rootElement, 1);
	mqsixLog("<< ParseFirstChild()\n");
}

void ParseLastChild(CciParser	*parser,
					   CciContext	*context,
					   CciElement	*currentElement)
{
	mqsixLog(">> ParseLastChild()\n");
}

void ParsePreviousSibling(CciParser		*parser,
						  CciContext	*context,
						  CciElement	*currentElement)
{
	mqsixLog(">> ParsePreviousSibling()\n");
}


void ParseNextSibling(CciParser	*parser,
						CciContext	*context,
						CciElement	*currentElement)
{
	mqsixLog(">> ParseNextSibling()\n");
}



///////////////////////////////////////////////////////////
// WriteBuffer
//
int WriteBuffer(CciParser	*parser,
				CciContext	*context)
{	
	mqsixLog(">> WriteBuffer()\n");
	// Do nothing ... this parser contributes NOTHING to the output!!
	mqsixLog("<< WriteBuffer()\n");
	return(0);
}


///////////////////////////////////////////////////////////
// ParserType
CciBool ParserType(CciParser *parser, CciContext *context)
{
	return(1);
}
///////////////////////////////////////////////////////////
// mqsixRegisterMessages
#ifdef _WIN32
void mqsixRegisterMessages(HMODULE hModule, char *messageSet)
{
	char	strFileName[512];
	char	strRegistryPath[512];
	HKEY	key;
	DWORD	iDisposition;
	DWORD	iValue;
	DWORD	iRc;

	GetModuleFileName(hModule, strFileName, sizeof(strFileName));
	mqsixLog("File name of DLL: %s\n", strFileName);
	sprintf(strRegistryPath,
		"SYSTEM\\CurrentControlSet\\Services\\Eventlog\\Application\\%s", messageSet);
	
	iRc = RegCreateKeyEx(HKEY_LOCAL_MACHINE,
		strRegistryPath,
		0,
		"",
		REG_OPTION_NON_VOLATILE,
		KEY_WRITE,
		NULL,
		&key,
		&iDisposition);
	if (iRc != ERROR_SUCCESS)
	{
		mqsixLog("Failed to RegCreateKeyEx: %d\n", iRc);
		return;
	}
	if (iDisposition == REG_CREATED_NEW_KEY)
	{
		RegSetValueEx(key, "EventMessageFile", 0, REG_SZ, (PBYTE)strFileName, strlen(strFileName) + 1);
		iValue = 7;
		RegSetValueEx(key, "TypesSupported", 0, REG_DWORD, (PBYTE)&iValue, sizeof(iValue));
	}
	RegCloseKey(key);
}
#endif

///////////////////////////////////////////////////////////
// bipGetParserFactory
void *bipGetParserFactory()
{
	CciFactory*     factoryObject;
	int             rc;
	static CPI_VFT  vftable;
	static char		*strParserName = "NameValue";
	
#ifdef _WIN32
	mqsixRegisterMessages(g_module, "MQSIV2_WKSPACE");
#endif
	mqsixLog("*** TreeAsIs Build: %s ***\n", __TIMESTAMP__);
	mqsixLog("%s", "Entered bipGetParserFactory()\n");
	/* Setup virtual function table with addresses of implementation functions */
	memset(&vftable, 0, sizeof(vftable));
	vftable.iFpCreateContext			= CreateContext;
	vftable.iFpDeleteContext			= DeleteContext;
	vftable.iFpParseBuffer				= ParseBuffer;
	vftable.iFpParseFirstChild			= ParseFirstChild;
	vftable.iFpParseLastChild			= ParseLastChild;
	vftable.iFpParseLeftSibling			= ParsePreviousSibling;
	vftable.iFpParseRightSibling		= ParseNextSibling;
	vftable.iFpWriteBuffer				= WriteBuffer;
	vftable.iFpIsHeaderParser			= ParserType;
	

	/* Create the message factory for this plug-in */
	factoryObject = cpiCreateParserFactory(&rc, L"WorkspaceFactory");
	if (rc != CCI_SUCCESS)
	{
		mqsixLog("Failed to create parser factory: rc=%d\n", rc);
	}
	if (factoryObject)
	{

		/* Define the classes of message supported by the factory */
		cpiDefineParserClass(&rc, factoryObject, L"WKSPACE", &vftable);
		if (rc != CCI_SUCCESS)
		{
			mqsixLog("Failed to create a parser: rc=%d, %s\n", rc, ErrorToString(rc));
		}
		else
		{
			mqsixLog("Created the parser called %s\n", "WKSPACE");
		}
	}
	else
	{
		mqsixLog("Failed to create factory object in bipGetParserFactory()\n");
		/* Error recovery */ 
	}

	/* Return address of this factory object to the broker */
	return(factoryObject);
}
